# ui/views.py
from django.template.loader import render_to_string
from django.shortcuts import render
from django.utils.safestring import mark_safe
from django.http import JsonResponse
import json
import ui.util as util
from collections import OrderedDict
import numpy as np

def index(request):
    schema = util.get_schema()
    #TODO: parse schema and render it to UI for side bar
    #TODO: make node_type.html display the first few results for that type
    #TODO: make the node_type.html page dynamic such that is traverses edges and populates the table
    return render(request, 'ui/index.html', {"schema": schema})



def node_type(request, node_type):
    # Will need to use ordereddicts
    schema = util.get_schema()
    # Make a request for the first few of that node type and populate the template
    node_examples = util.get_node_page(node_type)
    
    fields = [s['fields'] for s in schema['types'] if s['name'] == node_type][0] # a bit hacky
    fields = sorted(fields, key=lambda x: x['name'])
    new_node_examples = []
    for example in node_examples['all']:
        vals = OrderedDict()
        vals['uid'] = ''
        for field in fields:
            vals[field['name']] = ''
        for schema_field in vals.keys():
            field_vals = []
            lang_field = False
            for example_field, example_field_val in example.items():
                split_field_name = example_field.split("@")
                if example_field.split("@")[0] == (schema_field):
                    if type(example_field_val) == list:
                        field_vals.append(str(len(example_field_val)) + " edge(s)")
                    else:
                        if len(split_field_name) > 1:
                            field_vals.append({'lang': example_field.split("@")[1], 'value': str(example_field_val)[:30]})
                            lang_field = True
                        else:
                            field_vals.append(str(example_field_val)[:30])
            if len(field_vals) > 1 or lang_field:
                field_type = 'compound'
            else:
                field_type = 'simple'
            vals[schema_field] = {'field_type': field_type, 'field_vals': field_vals}
            metadata = {"instanceid": str(np.random.randint(100000))}
        new_node_examples.append({"data": vals, "metadata": metadata})
    return render(request, 'ui/node_type.html', {"schema": schema, "node_examples": new_node_examples, "fields": fields, "node_type": node_type})

def node_edge_type(request, node_uid, edge_type):
    # Will need to use ordereddicts
    schema = util.get_schema()

    # reorganize so that facets are part of json rather than embedded in strings
    edges = util.get_node_edge_type(node_uid, edge_type)

    if type(edges) == list:
        if type(edges[0]) == dict:
            if 'uid' in edges[0].keys():
                edge_nodes_p = 'nodes'
                new_edges = []
                for example in edges:
                    vals = OrderedDict()
                    #vals['uid'] = ''
                    for field in sorted(list(set([x.split("@")[0] for x in  example.keys()]))): # collapse across languages, exclude facets
                        vals[field] = ''
                    for schema_field in vals.keys():
                        field_vals = []
                        lang_field = False
                        for example_field, example_field_val in example.items():
                            split_field_name = example_field.split("@")
                            if example_field.split("@")[0] == (schema_field):
                                if type(example_field_val) == list:
                                    field_vals.append(str(len(example_field_val)) + " edge(s)")
                                else:
                                    if len(split_field_name) > 1:
                                        field_vals.append({'lang': example_field.split("@")[1], 'value': str(example_field_val)[:30]})
                                        lang_field = True
                                    else:
                                        field_vals.append(str(example_field_val)[:30])
                        if len(field_vals) > 1 or lang_field:
                            field_type = 'compound'
                        else:
                            field_type = 'simple'
                        vals[schema_field] = {'field_type': field_type, 'field_vals': field_vals}
                        metadata = {"instanceid": str(np.random.randint(100000))}
                    new_edges.append({"data": vals, "metadata": metadata})
                edges = new_edges
            else:
                edge_nodes_p = 'empty'
        else:
            edge_nodes_p = 'primitive'
    elif type(edges) == dict:
        # This happens with language tags
        edge_nodes_p = 'primitive'
        edges = [dict([(k,v)]) for k,v in edges.items()]
    else:
        edge_nodes_p = 'primitive'
        edges = [{edge_type: edges}]

    if edge_nodes_p == 'nodes':
        new_edges = []
        for edge in edges:
            facets = OrderedDict([(k,v) for k,v in edge["data"].items() if len(k.split('|')) > 1])
            edge["data"] = OrderedDict([(k,v) for k,v in edge["data"].items() if len(k.split('|')) == 1])
            edge_data = edge
            # edge_data = OrderedDict([(k,v) for k,v in sorted(edge.items()) if len(k.split('|')) == 1])
            new_edges.append({'facets': facets,
                              'data': edge_data})
    else:
        new_edges = []
        for edge in edges:
            facets = OrderedDict([(k,v) for k,v in sorted(edge.items()) if len(k.split('|')) > 1])
            edge_data = OrderedDict([(k,v) for k,v in sorted(edge.items()) if len(k.split('|')) == 1])
            new_edges.append({'facets': facets,
                              'data': edge_data})


    # new_edges = []
    # if (type(edges) == list and 'uid' in edges[0]) or ('uid' in edges):


    #     # should be a list or dict since a node shouldn't exist with just a uid and nothing else
    #     if type(edges) != list:
    #         edges = [edges]
    #     for edge in edges:
    #         facets = OrderedDict([(k,v) for k,v in sorted(edge.items()) if len(k.split('|')) > 1])
    #         edge_data = OrderedDict([(k,v) for k,v in sorted(edge.items()) if len(k.split('|')) == 1])
    #         new_edges.append({'facets': facets,
    #                           'data': edge_data})
    #     edge_nodes_p = 'nodes'
    # else:
    #     # should bve either a list or a singleton primitive
    #     if type(edges) != list:
    #         edges = [{edge_type: edges}]
            
    #     for edge in edges:
    #         facets = OrderedDict([(k,v) for k,v in sorted(edge.items()) if len(k.split('|')) > 1])
    #         edge_data = OrderedDict([(k,v) for k,v in sorted(edge.items()) if len(k.split('|')) == 1])
    #         new_edges.append({'facets': facets,
    #                           'data': edge_data})
    #     edge_nodes_p = 'primitive'
        
    html = render_to_string('ui/node_edge_type.html', {
        'schema': schema,
        'edge_nodes_p': edge_nodes_p,
        'edges': new_edges
    })
    data = {
        'schema': schema,
        'edge_nodes_p': edge_nodes_p,
        'edges': new_edges,
        'html': html 
    }
    return JsonResponse(data)

def get_edges(request, node, edge):
    schema = util.get_schema()
